classdef DataDecoder < matlab.System
    %#codegen
    
    %   Copyright 2012 The MathWorks, Inc.
    
    properties (Nontunable)
        TrainingFrameSize
        DataFrameSize
        TrainingSymbols
        ModulationOrder
        Period
    end
    
    properties (Access=private)
        pBuffer
        pCorrelator
        Xttilde
        Xttilde_inv
        NE
        L
        N_f
        Delta
        bigframesize
    end
    
    
    methods
        function obj = DataDecoder(varargin)
            setProperties(obj,nargin,varargin{:});
        end
    end
    
    methods (Access=protected)
        function setupImpl(obj, ~)
            obj.bigframesize=(obj.Period-1)*obj.DataFrameSize+obj.TrainingFrameSize;
            obj.pBuffer=dsp.Buffer(obj.bigframesize*3, 2*obj.bigframesize);
            obj.pCorrelator = dsp.Crosscorrelator;
            
            obj.NE=43;
            obj.L=5;
            obj.N_f=200;
            obj.Delta=150;
            
            
            for i=0:100+obj.L-1
                obj.Xttilde(:,i+1)=obj.TrainingSymbols(end-i:-1:end-i-obj.NE+1);
            end
            obj.Xttilde_inv=obj.Xttilde'*pinv(obj.Xttilde*obj.Xttilde');
            
        end
        
        function  [active_taps, mse_omp, mse_classical, mse_conv,h_omp,h_classical,h_conv] = stepImpl(obj, DataIn)
            BER=0;
            rxData = step(obj.pBuffer,DataIn);
            
            if rxData(obj.bigframesize+2)~=0
                z=abs(step(obj.pCorrelator,rxData(obj.bigframesize+(1:obj.bigframesize+obj.TrainingFrameSize-1)), obj.TrainingSymbols));
                z2=z(obj.TrainingFrameSize+(0:obj.bigframesize-1));
                [~, training_start]=max(z2);
            else
                training_start=901;
            end
            y=[];
            D=[];
            for f=1:obj.Period
                if (training_start>=(f-1)*obj.DataFrameSize+1) && (training_start<=f*obj.DataFrameSize)
                    y=rxData(obj.bigframesize+(f-1)*obj.DataFrameSize+obj.TrainingFrameSize+(0:-1:-obj.NE+1));
                    D=training_start-((f-1)*obj.DataFrameSize+1);
                    findex=f;
                    chift=obj.TrainingFrameSize+(f-1)*obj.DataFrameSize;
                end
            end
            
            if (training_start>=obj.Period*obj.DataFrameSize+1) && (training_start<=obj.bigframesize)
                y=rxData(obj.bigframesize+obj.Period*obj.DataFrameSize+obj.TrainingFrameSize+(0:-1:-obj.NE+1));
                D=training_start-(obj.Period*obj.DataFrameSize+1);
                findex=obj.Period+1;
                chift=obj.DataFrameSize;
            end
            
            
            Xt=obj.Xttilde(:,1+D:1+D+obj.L);
            Xt_inv=Xt'*pinv(Xt*Xt');
            h_conv=Xt_inv*y;
            
            h_omp=ompC(obj.Xttilde,y,obj.L+1);
            h_classical=obj.Xttilde_inv*y;
            h_conv=[zeros(D,1); h_conv; zeros(obj.DataFrameSize-D-1,1)];
            
            
            var_omp = (norm(y-obj.Xttilde*h_omp))^2/obj.NE;
            var_classical = (norm(y-obj.Xttilde*h_classical))^2/obj.NE;
            var_conv = (norm(y-obj.Xttilde*h_conv))^2/obj.NE;
            
            
            active_taps=obj.N_f;
            [w]=equalizer(obj,h_omp,var_omp,obj.N_f,obj.Delta,active_taps);
            xhat=filter(conj(w),1,rxData(obj.bigframesize+obj.Delta+(-obj.N_f+2:+obj.bigframesize)));
            xhat2=xhat(obj.N_f:end);
            
            load modulateddata
            
            if 1
                figure(1), plot(xhat2,'x')
                title('Constellation of Equalized symbols')
                figure(2)
                plot(sign(real(xhat2))-sign(real(circshift(ModulatedData,chift))),'x')
                xlabel('time')
                ylabel('Error')
                figure(3)
                plot(abs(h_omp),'x-')
                xlabel('Channel tap index')
                ylabel('Magnitute of the channel tap')
                title('Channel estimate using Proposed Method')
                
                figure(4)
                plot(abs(h_classical),'x-')
                xlabel('Channel tap index')
                ylabel('Magnitute of the channel tap')
                title('Channel estimate using Classical Method')
                figure(5)
                plot(abs(h_conv),'x-')
                xlabel('Channel tap index')
                ylabel('Magnitute of the channel tap')
                title('Channel estimate using Conventional Method')
            end
            
            active_taps=[1:5:30 obj.N_f];
            for aa=1:length(active_taps)
                w=equalizer(obj,h_omp,var_omp,obj.N_f,obj.Delta,active_taps(aa));
                xhat=filter(conj(w),1,rxData(obj.bigframesize+obj.Delta+(-obj.N_f+2:+obj.bigframesize)));
                xhat2=xhat(obj.N_f:end);
                e=xhat2-circshift(ModulatedData,chift);
                mse_omp(aa)=e'*e/length(e);
            end
            
            for aa=1:length(active_taps)
                w=equalizer(obj,h_classical,var_classical,obj.N_f,obj.Delta,active_taps(aa));
                xhat=filter(conj(w),1,rxData(obj.bigframesize+obj.Delta+(-obj.N_f+2:+obj.bigframesize)));
                xhat2=xhat(obj.N_f:end);
                e=xhat2-circshift(ModulatedData,chift);
                mse_classical(aa)=e'*e/length(e);
            end
            
            for aa=1:length(active_taps)
                w=equalizer(obj,h_conv,var_conv,obj.N_f,obj.Delta,active_taps(aa));
                xhat=filter(conj(w),1,rxData(obj.bigframesize+obj.Delta+(-obj.N_f+2:+obj.bigframesize)));
                xhat2=xhat(obj.N_f:end);
                e=xhat2-circshift(ModulatedData,chift);
                mse_conv(aa)=e'*e/length(e);
            end
            
            
            figure(12)
            clf
            semilogy(active_taps,mse_omp,'r')
            hold on
            semilogy(active_taps,mse_classical,'g')
            semilogy(active_taps,mse_conv,'k')
            xlabel('Number of active taps')
            ylabel('MSE')
            legend('Proposed Method','Classical Method','Conventional Method')

        end
        
        function resetImpl(obj)
            reset(obj.pBuffer);
        end
        
        function releaseImpl(obj)
            release(obj.pBuffer);
        end
        
    end
    
    methods (Access=private)
        
        
        function w=equalizer(obj,h,var_v,N_f,Delta,active_taps)
            v=length(h)-1;
            One_delta = [zeros(1,Delta) ,1 ,zeros(1,N_f +v-Delta-1)]';
            H = toeplitz([ h(1) zeros(1,(N_f-1)) ], [h.' zeros(1,(N_f-1))]);
            R_yy = (H*H')+ var_v*eye(N_f);
            r_delta = H * One_delta;
            [U,D] = eig(R_yy);
            w=ompC(D^0.5*U',D^(-0.5)*(U')*(r_delta),active_taps);
        end
        
        
        
    end
end

