classdef TransmitterObject < matlab.System  
% Generates the QPSK signal to be transmitted
    
    properties (Nontunable)
        Upsampling = 4;
        Period = 10;
        DataFrameLength = 100;
        TrainingFrameLength=147;
        TransmitterFilterCoefficients = 1;
        Channel=1;
    end
    
     properties (Access=private)
        pQPSKModulator 
        pTransmitterFilter
        pChannelFilter
        pTransmittedData
        pModulatedData
     end
    
    methods
        function obj = TransmitterObject(varargin)
            setProperties(obj,nargin,varargin{:});
        end
    end
    
    methods (Access=protected)
        function setupImpl(obj)
            obj.pQPSKModulator  = comm.QPSKModulator('BitInput',true, ...
                'PhaseOffset', pi/4);
            obj.pTransmitterFilter = dsp.FIRInterpolator(obj.Upsampling, ...
                obj.TransmitterFilterCoefficients);
            obj.pChannelFilter= dsp.FIRFilter;
            obj.pChannelFilter.Numerator =obj.Channel/norm(obj.Channel) ;
            NumberOfBits=((obj.Period-1)*obj.DataFrameLength+obj.TrainingFrameLength)*2;
            % Generates the data to be transmitted
            obj.pTransmittedData=(sign(randn(NumberOfBits,1))+1)/2;
            % Modulates the bits into QPSK symbols
            obj.pModulatedData = step(obj.pQPSKModulator, obj.pTransmittedData);
            % Saves QPSK symbols to be used at the receiver
            ModulatedData=obj.pModulatedData;
            save modulateddata ModulatedData
        end
        
        function transmittedSignal = stepImpl(obj)
            % Square root Raised Cosine Transmit Filter
            transmittedSignal = step(obj.pTransmitterFilter, obj.pModulatedData);
            transmittedSignal = step(obj.pChannelFilter, transmittedSignal);            
        end
        
        function resetImpl(obj)
            reset(obj.pQPSKModulator );
            reset(obj.pTransmitterFilter);
        end
        
        function releaseImpl(obj)
            release(obj.pQPSKModulator );
            release(obj.pTransmitterFilter);
        end
        
        function N = getNumInputsImpl(~)
            N = 0;
        end
    end
end

