function RX = receiver_init()
% Set simulation parameters
% General simulation parameters

RX.M = 4; % M-PSK alphabet size
RX.Upsampling = 4; % Upsampling factor
RX.Downsampling = 2; % Downsampling factor
RX.PostFilterOversampling=RX.Upsampling/RX.Downsampling;
RX.FrameSize = 9*100+147; % Number of modulated symbols per frame

% Generate square root raised cosine filter coefficients 
RX.RCFiltSpan = 10; % Filter span of Raised Cosine Tx Rx filters (in symbols)
RX.SquareRootRaisedCosineFilterOrder = RX.Upsampling*RX.RCFiltSpan;
RX.RollOff = 0.5;
% Square root raised cosine receive filter
hRxFilt = fdesign.decimator(RX.Upsampling/RX.Downsampling, ...
                'Square Root Raised Cosine', RX.Upsampling, ...
                'N,Beta', RX.SquareRootRaisedCosineFilterOrder, RX.RollOff);
hDRxFilt = design(hRxFilt, 'SystemObject', true);
RX.ReceiverFilterCoefficients = hDRxFilt.Numerator; 

% Rx parameters
K = 1;
A = 1/sqrt(2);
% Look into model for details for details of PLL parameter choice. Refer equation 7.30 of "Digital Communications - A Discrete-Time Approach" by Michael Rice. 
RX.PhaseErrorDetectorGain = 2*K*A^2+2*K*A^2; % K_p for Fine Frequency Compensation PLL, determined by 2KA^2 (for binary PAM), QPSK could be treated as two individual binary PAM
RX.PhaseRecoveryGain = 1; % K_0 for Fine Frequency Compensation PLL
RX.TimingErrorDetectorGain = 2.7*2*K*A^2+2.7*2*K*A^2; % K_p for Timing Recovery PLL, determined by 2KA^2*2.7 (for binary PAM), QPSK could be treated as two individual binary PAM, 2.7 is for raised cosine filter with roll-off factor 0.5
RX.TimingRecoveryGain = -1; % K_0 for Timing Recovery PLL, fixed due to modulo-1 counter structure
RX.CoarseCompFrequencyResolution =5; % 50; % Frequency resolution for coarse frequency compensation
RX.PhaseRecoveryLoopBandwidth = 0.005; %0.01; % Normalized loop bandwidth for fine frequency compensation
RX.PhaseRecoveryDampingFactor = 0.7; %1; % Damping Factor for fine frequency compensation
RX.TimingRecoveryLoopBandwidth = 0.01; % Normalized loop bandwidth for timing recovery
RX.TimingRecoveryDampingFactor = 1; % Damping Factor for timing recovery

